{ config, pkgs, inputs, ... }:

{
  imports =
    [ 
      ./cube-hardware-configuration.nix
      ./common.nix
    ];

  # Bootloader.
  boot.loader.systemd-boot.enable = false;
  boot.loader.grub = {
    enable = true;
    device = "nodev";
    efiSupport = true;
    useOSProber = true;
  };

  boot.loader.efi.canTouchEfiVariables = true;

  networking.hostName = "cube"; # Define your hostname.
  # networking.wireless.enable = true;  # Enables wireless support via wpa_supplicant.

  # Configure network proxy if necessary
  # networking.proxy.default = "http://user:password@proxy:port/";
  # networking.proxy.noProxy = "127.0.0.1,localhost,internal.domain";

  # Enable networking
  networking.networkmanager.enable = true;

  networking.hosts = {
     "127.0.0.1" = ["muhh.local"];
  };

  # Workaround for broken networkmanager/systemd thing
  # https://github.com/NixOS/nixpkgs/issues/180175#issuecomment-1658731959
  systemd.services.NetworkManager-wait-online = {
    serviceConfig = {
      ExecStart = [ "" "${pkgs.networkmanager}/bin/nm-online -q" ];
    };
  };

  security = {
    pam.services.swaylock = {};
    polkit.enable = true;
    rtkit.enable = true;
    sudo = {
      wheelNeedsPassword = false;
    };
  };
  services.avahi = {
    enable = true;
    nssmdns4 = true;
    openFirewall = true;
  };
  # services.borgmatic = {
  #   enable = true;
  #   configurations = {
  #     test = {
  #       source_directories = [ 
  #           "/home/muhh/muhhmory"
  #         ];
  #       repositories = [
  #         "ssh://u455876@u455876.your-storagebox.de/./test"
  #       ];
  #     };
  #     storage = {
  #         extraConfig = {
  #          ssh_command = "ssh -p 23 -i ~/.ssh/id_ed25519";
  #         };
  #       };
  #     };
  #   };
  # };
  services.envfs.enable = true;
  services.xserver = {
    xkb.layout = "eu";
    xkb.options = "compose:ralt";
    displayManager = {
      gdm = {
        enable = true;
      };
    };
    desktopManager.gnome.enable = true;
  };
  # services.displayManager = {
  #   # defaultSession = "niri";
  #   ly = {
  #     enable = false;
  #     settings = {};
  #   };
  # };

  services.flatpak.enable = true;
  services.fwupd.enable = true;
  services.openssh = {
    enable = true;
    settings.PasswordAuthentication = true;
    settings.PubkeyAuthentication = true;
  };
  services.pipewire = {
	  enable = true;
	  alsa.enable = true;
	  alsa.support32Bit = true;
	  pulse.enable = true;
    wireplumber.enable = true;
    # If you want to use JACK applications, uncomment this
    # jack.enable = true;
  };
  services.plex = {
    enable = true;
    dataDir = "/var/lib/plex";
    openFirewall = true;
    user = "plex";
    group = "plex";
  };
  services.jellyfin = {
    enable = true;
    openFirewall = true;
  };

  services.tailscale = {
    enable = true;
    authKeyFile = config.sops.secrets.tailscale_auth_key.path;
  };

  sops = {
    age.keyFile = "/home/muhh/.config/sops/age/keys.txt";
    defaultSopsFile = ../modules/sops/secrets/secrets.yaml;
    secrets.tailscale_auth_key = {};
  };

  # Define a user account. Don't forget to set a password with ‘passwd’.
  users.users.muhh = {
    isNormalUser = true;
    name = "muhh";
    description = "Markus Heurung";
    extraGroups = [ "audio" "docker" "libvirtd" "input" "networkmanager" "plugdev" "qemu-libvirtd" "video" "wheel"];
    shell = pkgs.fish;
    linger = true;
  };

  nixpkgs.config = {
    allowUnfree = true;
    allowUnfreePredicate = pkg: true;
  };

  environment.shells = with pkgs; [ bash fish zsh ];
  environment.systemPackages = with pkgs; [
    evolutionWithPlugins
    git
    home-manager
    jellyfin
    jellyfin-ffmpeg
    jellyfin-web
    mosh
    toolbox
    vim
    wget
  ];

  fonts = {
    enableDefaultPackages = true;
    packages = with pkgs; [
      atkinson-hyperlegible
      atkinson-monolegible
      nerd-fonts.iosevka
      # secret-config.packages.x86_64-linux.default
    ];
  };

  programs = {
    _1password.enable = true;
    _1password-gui = {
      enable = true;
      polkitPolicyOwners = ["muhh"];
    };
    fish.enable = true;
    gnupg.agent = {
      enable = true;
      enableSSHSupport = true;
    };
    light = {
      enable = true;
      brightnessKeys.enable = true;
    };
    hyprland.enable = true;
    sway.enable = true;
    # ssh = {
    #   pubkeyAcceptedKeyTypes = ["ssh-ed25519" "ssh-rsa"];
    #   hostKeyAlgorithms = ["ssh-ed25519" "ssh-rsa"];
    # };
  };
  virtualisation = {
    docker.enable = false;
    libvirtd = {
      enable = true;
    };
    podman = {
      enable = true;
      dockerCompat = true;
    };
  };

  xdg = {
    portal = {
      enable = true;
      extraPortals = with pkgs; [
        xdg-desktop-portal-wlr
        xdg-desktop-portal-gtk
      ];
    };
  };
  
  # Open ports in the firewall.
  # FIREWALL IS ENABLED BY DEFAULT - muhh
  networking.firewall.allowedTCPPorts = [ 19132 ];
  networking.firewall.allowedUDPPorts = [ 19132 ];
  # Or disable the firewall altogether.
  # networking.firewall.enable = false;

  # This value determines the NixOS release from which the default
  # settings for stateful data, like file locations and database versions
  # on your system were taken. It‘s perfectly fine and recommended to leave
  # this value at the release version of the first install of this system.
  # Before changing this value read the documentation for this option
  # (e.g. man configuration.nix or on https://nixos.org/nixos/options.html).
  system.stateVersion = "23.11"; # Did you read the comment?
}
